/************************************************************************
 *
 * Copyright (C) 2009-2021 IRCAD France
 * Copyright (C) 2012-2020 IHU Strasbourg
 *
 * This file is part of Sight.
 *
 * Sight is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Sight is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sight. If not, see <https://www.gnu.org/licenses/>.
 *
 ***********************************************************************/

#pragma once

#include "core/config.hpp"

#include <boost/utility.hpp>

#include <string>

namespace sight::core::runtime
{

namespace detail
{

namespace profile
{

/**
 * @brief   Starts a given module. Only the identifier of the module is passed to allow loading and unloading.
 */
class Initializer : public boost::noncopyable
{
public:

    /**
     * @brief       Constructor
     *
     * @param[in]   identifier  a string containing a module identifier
     */
    Initializer(const std::string& identifier);

    /**
     * @brief   Applies the initializer on the module.
     *
     * @remark  This method should be called directly.
     */
    void apply();

protected:

    const std::string m_identifier; ///< the module identifier
};

} // namespace profile

} // namespace detail

} // namespace sight::core::runtime
